==> cert_creation-1.3.15.241016-sources.jar: com/xdja/safeclient/certcreation/device/PKCS10ForSM2Utils.java <==
package com.xdja.safeclient.certcreation.device;

import android.text.TextUtils;
import android.util.Log;

import com.aircert.util.ModuleLog;
import com.xdja.cryptodev.CryptoDevManager;
import com.xdja.cryptodev.CryptoDevType;
import com.xdja.cryptodev.devapi.CryptoInstance;
import com.xdja.cryptodev.devapi.SM2Pubkey;
import com.xdja.safeclient.certcreation.AppConfig;
import com.xdja.safeclient.certcreation.config.CertRule;
import com.xdja.safeclient.certcreation.config.CertType;
import com.xdja.safeclient.certcreation.config.XDJAAlgParams;
import com.xdja.safeclient.certcreation.util.FidUtil;

import org.bouncycastle.asn1.ASN1ObjectIdentifier;
import org.bouncycastle.asn1.DERBitString;
import org.bouncycastle.asn1.DEROutputStream;
import org.bouncycastle.asn1.pkcs.CertificationRequest;
import org.bouncycastle.asn1.pkcs.CertificationRequestInfo;
import org.bouncycastle.asn1.x500.X500Name;
import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
import org.bouncycastle.asn1.x509.SubjectPublicKeyInfo;
import org.bouncycastle.operator.DefaultSignatureAlgorithmIdentifierFinder;
import org.bouncycastle.pkcs.PKCS10CertificationRequest;
import org.bouncycastle.util.encoders.Base64;

import java.io.ByteArrayOutputStream;
import java.io.IOException;

/**
 * Created by jff on 2018/11/14.
 */

public class PKCS10ForSM2Utils extends PKCS10Utils {

    private static final String TAG = "PKCS10ForSM2Utils";

    public static PKCS10ForSM2Utils pkcs10ForSM2Utils = new PKCS10ForSM2Utils();


    public PKCS10ForSM2Utils() {
    }

    public static PKCS10ForSM2Utils getInstance() {
        if (pkcs10ForSM2Utils == null) {
            pkcs10ForSM2Utils = new PKCS10ForSM2Utils();
        }
        return pkcs10ForSM2Utils;
    }

    /**
     * 生成SM2双证请求的p10
     *
     * @param type        卡类型
     * @param dn          主题
     * @param isApplyCert true-申请证书  false-更新证书信息
     * @return
     */
    @Override
    public String getP10RequestBC(CryptoDevType type, int containerNum, String dn, boolean isApplyCert) {
        ModuleLog.e("PKCS10ForSM2Utils getP10RequestBC");
        if (TextUtils.isEmpty(dn)) {
//            dn = "cn=李逍遥 452112199510214225,ou=00,ou=00,o=00,l=00,l=00,ST=41,C=CN";
            return "";
        }

        if (isApplyCert) {
            int keyResult = generateSM2Keys(type, containerNum);

            //TODO JFF 2018.7.20 需要跟zjc确认错误码处理
            if (keyResult != 0 && keyResult != -1) {
                return "errorCode" + CryptoDevManager.getInstance().getErrorText(keyResult);
            }
        }

        CryptoInstance cryptoInstance = DeviceWrapper.getCryptoInstance(type);
        if (cryptoInstance == null) {
            return "";
        }

        final byte pub[] = SM2PubKey2PubKey(type, containerNum);

        if (pub == null) {
            ModuleLog.e("生成SM2公钥异常");
            return "";
        }

        ASN1ObjectIdentifier eccOID = new ASN1ObjectIdentifier(XDJAAlgParams.OID_CURVE);//2.1是椭圆曲线的OID
        ASN1ObjectIdentifier sm2OID = new ASN1ObjectIdentifier(XDJAAlgParams.OID_SM2);//301是SM2曲线的OID
        SubjectPublicKeyInfo spi = new SubjectPublicKeyInfo(new AlgorithmIdentifier(eccOID, sm2OID), pub);
        //封装请求信息
        final X500Name x500Name = new X500Name(dn);
        CertificationRequestInfo cf1 = new CertificationRequestInfo(x500Name, spi, null);
        ByteArrayOutputStream outStream = new ByteArrayOutputStream();
        DEROutputStream derOutStream = new DEROutputStream(outStream);
        try {
            derOutStream.writeObject(cf1);
        } catch (IOException e) {
            e.printStackTrace();
        }
        byte[] certInfoBytes = outStream.toByteArray();

        //对CertificationRequestInfo进行签名
        byte[] signData = new byte[64];
        int[] signDataLen = new int[1];


        byte[][] keyPair = FidUtil.getKeyPairFid(type, containerNum);

        byte[] pubFid = keyPair[0];
        byte[] priFid = keyPair[1];
        int encodeRequestInfo = cryptoInstance.SM2Sign(pubFid, priFid, 1, certInfoBytes, certInfoBytes.length, signData, signDataLen);
        if (encodeRequestInfo != 0) {
            ModuleLog.e("P10  encodeRequestInfo  " + encodeRequestInfo+ "    " + CryptoDevManager.getInstance().getErrorText(encodeRequestInfo));
            return null;
        }
        DERBitString dbs = new DERBitString(signData);
        AlgorithmIdentifier signIdentifier = new DefaultSignatureAlgorithmIdentifierFinder().find("SM3WITHSM2");
        CertificationRequest cr = new CertificationRequest(cf1, signIdentifier, dbs);
        PKCS10CertificationRequest request = new PKCS10CertificationRequest(cr);
        try {
            byte[] buffer = request.getEncoded();

            String p10 = new String(Base64.encode(buffer));
//            ModuleLog.e("P10请求字符串 ： " + p10);
//            checkP10(type, p10);
            return p10;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "";
    }


    /**
     * 生成SM2签名公私钥对
     *
     * @param type
     * @return -1 设备不存在
     * 0 生成公私钥对成功
     * 其他  根据错误码返回错误信息
     */
    private int generateSM2Keys(CryptoDevType type, int containerNum) {
        ModuleLog.e("PKCS10ForSM2Utils generateSM2Keys");
        int genSm2KeysPairResult = -1;
        CryptoInstance cryptoInstance = DeviceWrapper.getCryptoInstance(type);
        if (cryptoInstance == null) {
            return genSm2KeysPairResult;
        }
        AppConfig module = AppConfig.getInstance();
        //此处添加验证PIN，是因为生成公私钥操作返回-15（没有权限）
        int pinResult = DeviceWrapper.verifyPin(type, containerNum, "");
        if (pinResult == 0) {
            genSm2KeysPairResult = genSM2KeyPair(cryptoInstance, type, containerNum);
        } else {
            return pinResult;
        }

        Log.e("jff", "160 PKCS10ForSM2Utils generateSM2Keys genSm2KeysPairResult : " + genSm2KeysPairResult);

        if (genSm2KeysPairResult == 2030 || genSm2KeysPairResult == 2041) {//容器不存在
            //2030对应卡库错误码-20——文件不存在
            // 2041对应卡库错误码-31——密钥文件不存在
            if (cryptoInstance.createContainer(module.getRole(type, containerNum), module.getDefaultConfig().getPin(), containerNum, "") == 0) {
                genSm2KeysPairResult = genSM2KeyPair(cryptoInstance, type, containerNum);
            } else {
                return genSm2KeysPairResult;
            }
        }
        cryptoInstance.close();
        return genSm2KeysPairResult;
    }

    private int genSM2KeyPair(CryptoInstance cryptoInstance, CryptoDevType type, int containerNum) {
        ModuleLog.e("PKCS10ForSM2Utils genSM2KeyPair");
        int genSm2KeysPairResult;
        if (AppConfig.getInstance().getCertConfig().getCertRule(type, containerNum).equals(CertRule.SINGLE)) {
            if (AppConfig.getInstance().getCertConfig().getCertType(type, containerNum).equals(CertType.SIGNING)
                    || TextUtils.isEmpty(AppConfig.getInstance().getCertConfig().getCertType(type, containerNum))) {
                genSm2KeysPairResult = cryptoInstance.genSignSM2KeyPair(containerNum);
            } else {
                genSm2KeysPairResult = cryptoInstance.genExchangeSM2KeyPair(containerNum);
            }
        } else {
            genSm2KeysPairResult = cryptoInstance.genSignSM2KeyPair(containerNum);
        }
        return genSm2KeysPairResult;
    }


    private byte[] SM2PubKey2PubKey(CryptoDevType type, int containerNum) {
        SM2Pubkey pubKey = new SM2Pubkey();
//        byte[] pubFid = new byte[]{0x00, 0x2D};
        byte[][] pubFid = FidUtil.getKeyPairFid(type, containerNum);
        CryptoInstance cryptoInstance = DeviceWrapper.getCryptoInstance(type);
        if (cryptoInstance == null) {
            return null;
        }
        int result = cryptoInstance.readSM2Pubkey(pubFid[0], pubKey);
        if (result != 0) {
            return null;
        }
        //获取公钥pub      SM2公钥byte[0]=4代表是SM2算法，紧跟着X,Y
        final byte pub[] = new byte[65];
        pub[0] = 4;
        System.arraycopy(pubKey.getX(), 0, pub, 1, pubKey.getX().length);
        System.arraycopy(pubKey.getY(), 0, pub, pubKey.getX().length + 1, pubKey.getY().length);
        ModuleLog.e("pub key : " + new String(Base64.encode(pub)));
        return pub;
    }



}

==> cert_creation-1.3.15.241227-sources.jar: com/xdja/safeclient/certcreation/device/PKCS10ForSM2Utils.java <==
package com.xdja.safeclient.certcreation.device;

import android.text.TextUtils;
import android.util.Log;
import android.util.Pair;

import com.aircert.util.ModuleLog;
import com.xdja.cryptodev.CryptoDevManager;
import com.xdja.cryptodev.CryptoDevType;
import com.xdja.cryptodev.devapi.CryptoInstance;
import com.xdja.cryptodev.devapi.SM2Pubkey;
//import com.xdja.multi.unitepin.jar.MultiChipUnitePinManager;
import com.xdja.safeclient.certcreation.AppConfig;
import com.xdja.safeclient.certcreation.config.CertRule;
import com.xdja.safeclient.certcreation.config.CertType;
import com.xdja.safeclient.certcreation.config.XDJAAlgParams;
import com.xdja.safeclient.certcreation.util.FidUtil;

import org.bouncycastle.asn1.ASN1ObjectIdentifier;
import org.bouncycastle.asn1.DERBitString;
import org.bouncycastle.asn1.DEROutputStream;
import org.bouncycastle.asn1.pkcs.CertificationRequest;
import org.bouncycastle.asn1.pkcs.CertificationRequestInfo;
import org.bouncycastle.asn1.x500.X500Name;
import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
import org.bouncycastle.asn1.x509.SubjectPublicKeyInfo;
import org.bouncycastle.operator.DefaultSignatureAlgorithmIdentifierFinder;
import org.bouncycastle.pkcs.PKCS10CertificationRequest;
import org.bouncycastle.util.encoders.Base64;

import java.io.ByteArrayOutputStream;
import java.io.IOException;

/**
 * Created by jff on 2018/11/14.
 */

public class PKCS10ForSM2Utils extends PKCS10Utils {

    private static final String TAG = "PKCS10ForSM2Utils";

    public static PKCS10ForSM2Utils pkcs10ForSM2Utils = new PKCS10ForSM2Utils();


    public PKCS10ForSM2Utils() {
    }

    public static PKCS10ForSM2Utils getInstance() {
        if (pkcs10ForSM2Utils == null) {
            pkcs10ForSM2Utils = new PKCS10ForSM2Utils();
        }
        return pkcs10ForSM2Utils;
    }

    /**
     * 生成SM2双证请求的p10
     *
     * @param type        卡类型
     * @param dn          主题
     * @param isApplyCert true-申请证书  false-更新证书信息
     * @return
     */
    @Override
    public String getP10RequestBC(CryptoDevType type, int containerNum, String dn, boolean isApplyCert) {
        ModuleLog.e("PKCS10ForSM2Utils getP10RequestBC");
        if (TextUtils.isEmpty(dn)) {
//            dn = "cn=李逍遥 452112199510214225,ou=00,ou=00,o=00,l=00,l=00,ST=41,C=CN";
            return "";
        }

        if (isApplyCert) {
            int keyResult = generateSM2Keys(type, containerNum);

            //TODO JFF 2018.7.20 需要跟zjc确认错误码处理
            if (keyResult != 0 && keyResult != -1) {
                return "errorCode" + CryptoDevManager.getInstance().getErrorText(keyResult);
            }
        }

        CryptoInstance cryptoInstance = DeviceWrapper.getCryptoInstance(type);
        if (cryptoInstance == null) {
            return "";
        }

        final byte pub[] = SM2PubKey2PubKey(type, containerNum);

        if (pub == null) {
            ModuleLog.e("生成SM2公钥异常");
            return "";
        }

        ASN1ObjectIdentifier eccOID = new ASN1ObjectIdentifier(XDJAAlgParams.OID_CURVE);//2.1是椭圆曲线的OID
        ASN1ObjectIdentifier sm2OID = new ASN1ObjectIdentifier(XDJAAlgParams.OID_SM2);//301是SM2曲线的OID
        SubjectPublicKeyInfo spi = new SubjectPublicKeyInfo(new AlgorithmIdentifier(eccOID, sm2OID), pub);
        //封装请求信息
        final X500Name x500Name = new X500Name(dn);
        CertificationRequestInfo cf1 = new CertificationRequestInfo(x500Name, spi, null);
        ByteArrayOutputStream outStream = new ByteArrayOutputStream();
        DEROutputStream derOutStream = new DEROutputStream(outStream);
        try {
            derOutStream.writeObject(cf1);
        } catch (IOException e) {
            e.printStackTrace();
        }
        byte[] certInfoBytes = outStream.toByteArray();

        //对CertificationRequestInfo进行签名
        byte[] signData = new byte[64];
        int[] signDataLen = new int[1];


        byte[][] keyPair = FidUtil.getKeyPairFid(type, containerNum);

        byte[] pubFid = keyPair[0];
        byte[] priFid = keyPair[1];
        int encodeRequestInfo = cryptoInstance.SM2Sign(pubFid, priFid, 1, certInfoBytes, certInfoBytes.length, signData, signDataLen);
        if (encodeRequestInfo != 0) {
            ModuleLog.e("P10  encodeRequestInfo  " + encodeRequestInfo+ "    " + CryptoDevManager.getInstance().getErrorText(encodeRequestInfo));
            return null;
        }
        DERBitString dbs = new DERBitString(signData);
        AlgorithmIdentifier signIdentifier = new DefaultSignatureAlgorithmIdentifierFinder().find("SM3WITHSM2");
        CertificationRequest cr = new CertificationRequest(cf1, signIdentifier, dbs);
        PKCS10CertificationRequest request = new PKCS10CertificationRequest(cr);
        try {
            byte[] buffer = request.getEncoded();

            String p10 = new String(Base64.encode(buffer));
//            ModuleLog.e("P10请求字符串 ： " + p10);
//            checkP10(type, p10);
            return p10;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "";
    }


    /**
     * 生成SM2签名公私钥对
     *
     * @param type
     * @return -1 设备不存在
     * 0 生成公私钥对成功
     * 其他  根据错误码返回错误信息
     */
    private int generateSM2Keys(CryptoDevType type, int containerNum) {
        ModuleLog.e("PKCS10ForSM2Utils generateSM2Keys");
        int genSm2KeysPairResult = -1;
        CryptoInstance cryptoInstance = DeviceWrapper.getCryptoInstance(type);
        if (cryptoInstance == null) {
            return genSm2KeysPairResult;
        }
        AppConfig module = AppConfig.getInstance();
        //此处添加验证PIN，是因为生成公私钥操作返回-15（没有权限）
        int pinResult = DeviceWrapper.verifyPin(type, containerNum, "");
        if (pinResult == 0) {
            genSm2KeysPairResult = genSM2KeyPair(cryptoInstance, type, containerNum);
        } else {
            return pinResult;
        }

        Log.e("jff", "160 PKCS10ForSM2Utils generateSM2Keys genSm2KeysPairResult : " + genSm2KeysPairResult);

        if (genSm2KeysPairResult == 2030 || genSm2KeysPairResult == 2041) {//容器不存在
            //2030对应卡库错误码-20——文件不存在
            // 2041对应卡库错误码-31——密钥文件不存在

            String  pin = "111111";
            //Pair<Integer, String> pinPair = MultiChipUnitePinManager.getInstance().getPin(AppConfig.getInstance().getContext(), String.valueOf(cryptoInstance.getCardId()), module.getRole(type, containerNum));
            //if (pinPair.first == 0){
            //    ModuleLog.d("315 DeviceWrapper verifyPin : ");
            //    pin = pinPair.second ;
            //} else {
            //    ModuleLog.d("319 DeviceWrapper verifyPin : 没有获取到默认pin，设置pin");
            //    pin = "111111";
            //}


            if (cryptoInstance.createContainer(module.getRole(type, containerNum), pin, containerNum, "") == 0) {
                genSm2KeysPairResult = genSM2KeyPair(cryptoInstance, type, containerNum);
            } else {
                return genSm2KeysPairResult;
            }
        }
        cryptoInstance.close();
        return genSm2KeysPairResult;
    }

    private int genSM2KeyPair(CryptoInstance cryptoInstance, CryptoDevType type, int containerNum) {
        ModuleLog.e("PKCS10ForSM2Utils genSM2KeyPair");
        int genSm2KeysPairResult;
        if (AppConfig.getInstance().getCertConfig().getCertRule(type, containerNum).equals(CertRule.SINGLE)) {
            if (AppConfig.getInstance().getCertConfig().getCertType(type, containerNum).equals(CertType.SIGNING)
                    || TextUtils.isEmpty(AppConfig.getInstance().getCertConfig().getCertType(type, containerNum))) {
                genSm2KeysPairResult = cryptoInstance.genSignSM2KeyPair(containerNum);
            } else {
                genSm2KeysPairResult = cryptoInstance.genExchangeSM2KeyPair(containerNum);
            }
        } else {
            genSm2KeysPairResult = cryptoInstance.genSignSM2KeyPair(containerNum);
        }
        return genSm2KeysPairResult;
    }


    private byte[] SM2PubKey2PubKey(CryptoDevType type, int containerNum) {
        SM2Pubkey pubKey = new SM2Pubkey();
//        byte[] pubFid = new byte[]{0x00, 0x2D};
        byte[][] pubFid = FidUtil.getKeyPairFid(type, containerNum);
        CryptoInstance cryptoInstance = DeviceWrapper.getCryptoInstance(type);
        if (cryptoInstance == null) {
            return null;
        }
        int result = cryptoInstance.readSM2Pubkey(pubFid[0], pubKey);
        if (result != 0) {
            return null;
        }
        //获取公钥pub      SM2公钥byte[0]=4代表是SM2算法，紧跟着X,Y
        final byte pub[] = new byte[65];
        pub[0] = 4;
        System.arraycopy(pubKey.getX(), 0, pub, 1, pubKey.getX().length);
        System.arraycopy(pubKey.getY(), 0, pub, pubKey.getX().length + 1, pubKey.getY().length);
        ModuleLog.e("pub key : " + new String(Base64.encode(pub)));
        return pub;
    }



}
